from itertools import count
from stringcase import titlecase
from normality import slugify

from memorious.helpers.asp import ViewForm  # noqa
from memorious.helpers.dates import parse_date  # noqa
from memorious.helpers.key import make_id  # noqa


def convert_snakecase(name):
    if name.upper() != name:
        name = titlecase(name)
    return slugify(name, sep='_')


def soviet_checksum(code):
    """Courtesy of Sir Vlad Lavrov."""
    def sum_digits(code, offset=1):
        total = 0
        for digit, index in zip(code[:7], count(offset)):
            total += int(digit) * index
        summed = (total / 11 * 11)
        return total - summed

    check = sum_digits(code, 1)
    if check == 10:
        check = sum_digits(code, 3)
        if check == 10:
            return code + '0'
    return code + str(check)


def search_results_total(html, xpath, check, delimiter):
    """ Get the total number of results from the DOM of a search index. """
    for container in html.findall(xpath):
        if check in container.findtext('.'):
            text = container.findtext('.').split(delimiter)
            total = int(text[-1].strip())
            return total


def search_results_last_url(html, xpath, label):
    """ Get the URL of the 'last' button in a search results listing. """
    for container in html.findall(xpath):
        if container.text_content().strip() == label:
            return container.find('.//a').get('href')



import logging
import requests
from requests.packages.urllib3.exceptions import InsecureRequestWarning

requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
logging.getLogger('requests').setLevel(logging.WARNING)
logging.getLogger('zeep').setLevel(logging.WARNING)
logging.getLogger('httpstream').setLevel(logging.WARNING)


# Register signal handlers
from . import signals  # noqa



from six.moves.urllib.parse import urljoin
from requests.exceptions import RequestException

from memorious.helpers.rule import Rule
from memorious.util import make_key


def fetch(context, data):
    """Do an HTTP GET on the ``url`` specified in the inbound data."""
    url = data.get('url')
    attempt = data.pop('retry_attempt', 1)
    try:
        result = context.http.get(url, lazy=True)
        rules = context.get('rules', {'match_all': {}})
        if not Rule.get_rule(rules).apply(result):
            context.log.info('Fetch skip: %r', result.url)
            return

        if not result.ok:
            err = (result.url, result.status_code)
            context.emit_warning("Fetch fail [%s]: HTTP %s" % err)
            if not context.params.get('emit_errors', False):
                return
        else:
            context.log.info("Fetched [%s]: %r",
                             result.status_code,
                             result.url)

        data.update(result.serialize())
        if url != result.url:
            tag = make_key(context.run_id, url)
            context.set_tag(tag, None)
        context.emit(data=data)
    except RequestException as ce:
        retries = int(context.get('retry', 3))
        if retries >= attempt:
            context.log.warn("Retry: %s (error: %s)", url, ce)
            data['retry_attempt'] = attempt + 1
            context.recurse(data=data, delay=2 ** attempt)
        else:
            context.emit_warning("Fetch fail [%s]: %s" % (url, ce))


def dav_index(context, data):
    """List files in a WebDAV directory."""
    # This is made to work with ownCloud/nextCloud, but some rumor has
    # it they are "standards compliant" and it should thus work for
    # other DAV servers.
    url = data.get('url')
    result = context.http.request('PROPFIND', url)
    for resp in result.xml.findall('./{DAV:}response'):
        href = resp.findtext('./{DAV:}href')
        if href is None:
            continue

        rurl = urljoin(url, href)
        rdata = data.copy()
        rdata['url'] = rurl
        rdata['foreign_id'] = rurl
        if rdata['url'] == url:
            continue

        if resp.find('.//{DAV:}collection') is not None:
            rdata['parent_foreign_id'] = rurl
            context.log.info("Fetching contents of folder: %s" % rurl)
            context.recurse(data=rdata)
        else:
            rdata['parent_foreign_id'] = url

        # Do GET requests on the urls
        fetch(context, rdata)


def session(context, data):
    """Set some HTTP parameters for all subsequent requests.

    This includes ``user`` and ``password`` for HTTP basic authentication,
    and ``user_agent`` as a header.
    """
    context.http.reset()

    user = context.get('user')
    password = context.get('password')

    if user is not None and password is not None:
        context.http.session.auth = (user, password)

    user_agent = context.get('user_agent')
    if user_agent is not None:
        context.http.session.headers['User-Agent'] = user_agent

    referer = context.get('url')
    if referer is not None:
        context.http.session.headers['Referer'] = referer

    proxy = context.get('proxy')
    if proxy is not None:
        proxies = {'http': proxy, 'https': proxy}
        context.http.session.proxies = proxies

    # Explictly save the session because no actual HTTP requests were made.
    context.http.save()
    context.emit(data=data)



import logging

from memorious import settings
from memorious.core import init_memorious
from memorious.ui.views import app, sentry


init_memorious()
if settings.SENTRY_DSN:
    sentry.init_app(app,
                    dsn=settings.SENTRY_DSN,
                    logging=True,
                    level=logging.ERROR)


if __name__ == '__main__':
    app.run(host='0.0.0.0', port=4000, debug=True)



import datetime

from memorious.reporting import log_operation_start, log_operation_end


class TestReporting(object):
    def test_operation_reporting(self, crawler, context):
        log_operation_start(context)
        assert crawler.latest_runid == context.run_id
        assert len(list(crawler.runs)) == 1
        assert crawler.op_count == 1
        assert isinstance(crawler.last_run, datetime.datetime)
        log_operation_end(context)
        crawler.flush()
        assert crawler.op_count == 0
        assert len(list(crawler.runs)) == 0
